/*
* Copyright (C) 2011-2016 AirDC++ Project
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef DCPLUSPLUS_DCPP_WEBUSER_API_H
#define DCPLUSPLUS_DCPP_WEBUSER_API_H

#include <web-server/stdinc.h>

#include <api/ApiModule.h>
#include <api/WebUserUtils.h>
#include <api/common/ListViewController.h>

#include <web-server/WebUserManagerListener.h>

namespace webserver {
	class WebUserApi : public SubscribableApiModule, private WebUserManagerListener {
	public:
		WebUserApi(Session* aSession);
		~WebUserApi();

		int getVersion() const noexcept {
			return 0;
		}
	private:
		api_return handleGetUsers(ApiRequest& aRequest);
		api_return handleAddUser(ApiRequest& aRequest);
		api_return handleUpdateUser(ApiRequest& aRequest);
		api_return handleRemoveUser(ApiRequest& aRequest);
		void parseUser(WebUserPtr& aUser, const json& j, bool aIsNew);

		void on(WebUserManagerListener::UserAdded, const WebUserPtr& aUser) noexcept;
		void on(WebUserManagerListener::UserUpdated, const WebUserPtr& aUser) noexcept;
		void on(WebUserManagerListener::UserRemoved, const WebUserPtr& aUser) noexcept;

		typedef ListViewController<WebUserPtr, WebUserUtils::PROP_LAST> RootView;
		RootView view;

		WebUserManager& um;
		WebUserList getUsers() const noexcept;
	};
}

#endif